/**
 * Stateless implementations for evaluating more complex operators, and helpers
 * to operator evaluation.
 *
 * @author Igor Konnov, Gabriela Moreira
 *
 * @module
 */
import { Either } from '@sweet-monads/either';
import { Computable, EvaluationResult } from '../runtime';
import { RuntimeValue } from './runtimeValue';
import { List } from 'immutable';
export declare function applyFun(sourceId: bigint, fun: (..._args: RuntimeValue[]) => EvaluationResult, args: Computable[]): Computable;
export declare function applyFold(order: 'fwd' | 'rev', args: Computable[]): Computable;
export declare function getListElem(list: List<RuntimeValue>, idx: number): EvaluationResult;
export declare function updateList(list: List<RuntimeValue>, idx: number, value: RuntimeValue): EvaluationResult;
export declare function sliceList(list: List<RuntimeValue>, start: number, end: number): Either<never, RuntimeValue>;
export declare function applyBoolOp(defaultValue: RuntimeValue, shortCircuit: (no: number, r: boolean) => RuntimeValue | undefined, args: Computable[]): Computable;
/**
 * A generalized application of a one-argument Callable to a set-like
 * runtime value, as required by `exists`, `forall`, `map`, and `filter`.
 *
 * This method expects `compStack` to look like follows:
 *
 *  - `(top)` translated lambda, as `Callable`.
 *
 *  - `(top - 1)`: a set-like value to iterate over, as `Computable`.
 *
 * The method evaluates the Callable for each element of the iterable value
 * and either produces `none`, if evaluation failed for one of the elements,
 * or it applies `mapResultAndElems` to the pairs that consists of the Callable
 * result and the original element of the iterable value.
 * The final result is stored on the stack.
 */
export declare function mapLambdaThenReduce(sourceId: bigint, reduceFunction: (_array: Array<[RuntimeValue, RuntimeValue]>) => RuntimeValue, args: Computable[]): Computable;
